@@ -343,6 +343,7 @@ typedef enum _snd_pcm_tstamp {
 	SND_PCM_TSTAMP_LAST = SND_PCM_TSTAMP_ENABLE
 } snd_pcm_tstamp_t;
 
+/** PCM timestamp type */
 typedef enum _snd_pcm_tstamp_type {
 	SND_PCM_TSTAMP_TYPE_GETTIMEOFDAY = 0,	/**< gettimeofday equivalent */
 	SND_PCM_TSTAMP_TYPE_MONOTONIC,	/**< posix_clock_monotonic equivalent */
@@ -350,6 +351,7 @@ typedef enum _snd_pcm_tstamp_type {
 	SND_PCM_TSTAMP_TYPE_LAST = SND_PCM_TSTAMP_TYPE_MONOTONIC_RAW,
 } snd_pcm_tstamp_type_t;
 
+/** PCM audio timestamp type */
 typedef enum _snd_pcm_audio_tstamp_type {
 	/**
 	 * first definition for backwards compatibility only,
@@ -364,24 +366,22 @@ typedef enum _snd_pcm_audio_tstamp_type {
 	SND_PCM_AUDIO_TSTAMP_TYPE_LAST = SND_PCM_AUDIO_TSTAMP_TYPE_LINK_SYNCHRONIZED
 } snd_pcm_audio_tstamp_type_t;
 
+/** PCM audio timestamp config */
 typedef struct _snd_pcm_audio_tstamp_config {
 	/* 5 of max 16 bits used */
-	unsigned int type_requested:4;
-	unsigned int report_delay:1; /* add total delay to A/D or D/A */
+	unsigned int type_requested:4; /**< requested audio tstamp type */
+	unsigned int report_delay:1; /**< add total delay to A/D or D/A */
 } snd_pcm_audio_tstamp_config_t;
 
+/** PCM audio timestamp report */
 typedef struct _snd_pcm_audio_tstamp_report {
 	/* 6 of max 16 bits used for bit-fields */
 
-	/* for backwards compatibility */
-	unsigned int valid:1;
+	unsigned int valid:1; /**< for backwards compatibility */
+	unsigned int actual_type:4; /**< actual type if hardware could not support requested timestamp */
 
-	/* actual type if hardware could not support requested timestamp */
-	unsigned int actual_type:4;
-
-	/* accuracy represented in ns units */
-	unsigned int accuracy_report:1; /* 0 if accuracy unknown, 1 if accuracy field is valid */
-	unsigned int accuracy; /* up to 4.29s, will be packed in separate field  */
+	unsigned int accuracy_report:1; /**< 0 if accuracy unknown, 1 if accuracy field is valid */
+	unsigned int accuracy; /**< up to 4.29s in ns units, will be packed in separate field  */
 } snd_pcm_audio_tstamp_report_t;
 
 /** Unsigned frames quantity */
@@ -393,6 +393,8 @@ typedef long snd_pcm_sframes_t;
 #define SND_PCM_NONBLOCK		0x00000001
 /** Async notification (flag for open mode) \hideinitializer */
 #define SND_PCM_ASYNC			0x00000002
+/** Return EINTR instead blocking (wait operation) */
+#define SND_PCM_EINTR			0x00000080
 /** In an abort state (internal, not allowed for open) */
 #define SND_PCM_ABORT			0x00008000
 /** Disable automatic (but not forced!) rate resamplinig */
@@ -498,6 +500,13 @@ typedef union _snd_pcm_sync_id {
 	unsigned int id32[4];
 } snd_pcm_sync_id_t;
 
+/** Infinite wait for snd_pcm_wait() */
+#define SND_PCM_WAIT_INFINITE		(-1)
+/** Wait for next i/o in snd_pcm_wait() */
+#define SND_PCM_WAIT_IO			(-10001)
+/** Wait for drain in snd_pcm_wait() */
+#define SND_PCM_WAIT_DRAIN		(-10002)
+
 /** #SND_PCM_TYPE_METER scope handle */
 typedef struct _snd_pcm_scope snd_pcm_scope_t;
 
@@ -722,6 +731,7 @@ int snd_pcm_hw_params_is_half_duplex(const snd_pcm_hw_params_t *params);
 int snd_pcm_hw_params_is_joint_duplex(const snd_pcm_hw_params_t *params);
 int snd_pcm_hw_params_can_sync_start(const snd_pcm_hw_params_t *params);
 int snd_pcm_hw_params_can_disable_period_wakeup(const snd_pcm_hw_params_t *params);
+int snd_pcm_hw_params_is_perfect_drain(const snd_pcm_hw_params_t *params);
 int snd_pcm_hw_params_supports_audio_wallclock_ts(const snd_pcm_hw_params_t *params); /* deprecated, use audio_ts_type */
 int snd_pcm_hw_params_supports_audio_ts_type(const snd_pcm_hw_params_t *params, int type);
 int snd_pcm_hw_params_get_rate_numden(const snd_pcm_hw_params_t *params,
@@ -821,6 +831,8 @@ int snd_pcm_hw_params_set_export_buffer(snd_pcm_t *pcm, snd_pcm_hw_params_t *par
 int snd_pcm_hw_params_get_export_buffer(snd_pcm_t *pcm, snd_pcm_hw_params_t *params, unsigned int *val);
 int snd_pcm_hw_params_set_period_wakeup(snd_pcm_t *pcm, snd_pcm_hw_params_t *params, unsigned int val);
 int snd_pcm_hw_params_get_period_wakeup(snd_pcm_t *pcm, snd_pcm_hw_params_t *params, unsigned int *val);
+int snd_pcm_hw_params_set_drain_silence(snd_pcm_t *pcm, snd_pcm_hw_params_t *params, unsigned int val);
+int snd_pcm_hw_params_get_drain_silence(snd_pcm_t *pcm, snd_pcm_hw_params_t *params, unsigned int *val);
 
 int snd_pcm_hw_params_get_period_time(const snd_pcm_hw_params_t *params, unsigned int *val, int *dir);
 int snd_pcm_hw_params_get_period_time_min(const snd_pcm_hw_params_t *params, unsigned int *val, int *dir);
