@@ -255,6 +255,44 @@ int snd_seq_set_client_event_filter(snd_seq_t *seq, int event_type)
 	return snd_seq_set_client_info(seq, &info);
 }
 
+/**
+ * \brief set client MIDI protocol version
+ * \param seq sequencer handle
+ * \param midi_version MIDI protocol version to set
+ * \return 0 on success or negative error code
+ *
+ * \sa snd_seq_set_client_info()
+ */
+int snd_seq_set_client_midi_version(snd_seq_t *seq, int midi_version)
+{
+	snd_seq_client_info_t info;
+	int err;
+
+	if ((err = snd_seq_get_client_info(seq, &info)) < 0)
+		return err;
+	snd_seq_client_info_set_midi_version(&info, midi_version);
+	return snd_seq_set_client_info(seq, &info);
+}
+
+/**
+ * \brief enable/disable client's automatic conversion of UMP/legacy events
+ * \param seq sequencer handle
+ * \param enable 0 or 1 to disable/enable the conversion
+ * \return 0 on success or negative error code
+ *
+ * \sa snd_seq_set_client_info()
+ */
+int snd_seq_set_client_ump_conversion(snd_seq_t *seq, int enable)
+{
+	snd_seq_client_info_t info;
+	int err;
+
+	if ((err = snd_seq_get_client_info(seq, &info)) < 0)
+		return err;
+	snd_seq_client_info_set_ump_conversion(&info, enable);
+	return snd_seq_set_client_info(seq, &info);
+}
+
 /**
  * \brief change the output pool size of the given client
  * \param seq sequencer handle
@@ -388,25 +426,43 @@ int snd_seq_sync_output_queue(snd_seq_t *seq)
  */
 int snd_seq_parse_address(snd_seq_t *seq, snd_seq_addr_t *addr, const char *arg)
 {
-	char *p;
-	int client, port;
+	char *p, *buf;
+	const char *s;
+	char c;
+	long client, port = 0;
 	int len;
 
 	assert(addr && arg);
 
-	if ((p = strpbrk(arg, ":.")) != NULL) {
-		if ((port = atoi(p + 1)) < 0)
-			return -EINVAL;
-		len = (int)(p - arg); /* length of client name */
+	c = *arg;
+	if (c == '"' || c == '\'') {
+		s = ++arg;
+		while (*s && *s != c) s++;
+		len = s - arg;
+		if (*s)
+			s++;
+		if (*s) {
+			if (*s != '.' && *s != ':')
+				return -EINVAL;
+			if ((port = atoi(s + 1)) < 0)
+				return -EINVAL;
+		}
 	} else {
-		port = 0;
-		len = strlen(arg);
+		if ((p = strpbrk(arg, ":.")) != NULL) {
+			if ((port = atoi(p + 1)) < 0)
+				return -EINVAL;
+			len = (int)(p - arg); /* length of client name */
+		} else {
+			len = strlen(arg);
+		}
 	}
+	if (len == 0)
+		return -EINVAL;
+	buf = alloca(len + 1);
+	strncpy(buf, arg, len);
+	buf[len] = '\0';
 	addr->port = port;
-	if (isdigit(*arg)) {
-		client = atoi(arg);
-		if (client < 0)
-			return -EINVAL;
+	if (safe_strtol(buf, &client) == 0) {
 		addr->client = client;
 	} else {
 		/* convert from the name */
